@@ -267,32 +267,39 @@ def omit_deep(omit_props, dct):
 
 
 @curry
-def map_deep(map_props, dct):
+def map_with_obj_deep(f, dct):
     """
     Implementation of map that recurses. This tests the same keys at every level of dict and in lists
-    :param map_props: prop to unary function to map the value of a prop. The props are evaluated at every level
-    of dct
+    :param f: 2-ary function expecting a key and value and returns a modified value
     :param dct: Dict for deep processing
     :return: Modified dct with matching props mapped
     """
+    return _map_deep(lambda k, v: [k, f(k,v)], dct)
 
-    map_deep_partial = map_deep(map_props)
 
-    def test(key, value):
-        return prop_or(always(value), key, map_props)(value)
+@curry
+def map_keys_deep(f, dct):
+    """
+    Implementation of map that recurses. This tests the same keys at every level of dict and in lists
+    :param f: 2-ary function expecting a key and value and returns a modified key
+    :param dct: Dict for deep processing
+    :return: Modified dct with matching props mapped
+    """
+    return _map_deep(lambda k, v: [f(k, v), v], dct)
+
+def _map_deep(f, dct):
+    """
+    Used by map_deep and map_keys_deep
+    :param map_props:
+    :param f: Expects a key and value and returns a pair
+    :param dct:
+    :return:
+    """
 
     if isinstance(dict, dct):
-        # Filter out keys and then recurse on each value
-        return map_dict(map_deep_partial, compact_dict(
-            map_with_obj(
-                # Lambda calls map_props[key](value) if map_props[key] exists, else returns value
-                lambda key, value: test(key, value),
-                dct
-            )
-        ))
-    if isinstance((list, tuple), dct):
-        # run map_deep on each value
-        return map(map_deep_partial, dct)
+        return map_key_values(lambda k, v: f(k, _map_deep(f, v)), dct)
+    elif isinstance((list, tuple), dct):
+        return map(lambda iv: f(iv[0], _map_deep(f, iv[1])), enumerate(dct))
     # scalar
     return dct
 
@@ -340,7 +347,7 @@ def dict_matches_params_deep(params_dct, dct):
             # Recurse on each value if there is a corresponding filter_dct[key]. If not we pass
             lambda key, value: recurse_if_param_exists(params_dct, key, value),
             # We shallow merge, giving dct priority with (hopefully) unmatchable values
-            merge(map_with_obj(lambda k, v: 1/(-e * pi), params_dct), dct)
+            merge(map_with_obj(lambda k, v: 1 / (-e * pi), params_dct), dct)
         )
 
     if isinstance((list, tuple), dct):
@@ -400,10 +407,11 @@ def map_with_obj(f, dct):
     return f_dict
 
 
+@curry
 def map_with_obj_to_values(f, dct):
     """
         Like map_with_obj but just returns the mapped values an array and disgards the keys
-    :param f: Called wiht a key and value
+    :param f: Called with a key and value
     :param dct:
     :return {list}: values are the mapped value
     """
@@ -435,6 +443,20 @@ def map_keys(f, dct):
     return f_dict
 
 
+@curry
+def map_keys_with_obj(f, dct):
+    """
+        Calls f with each key and value of dct, possibly returning a modified key. Values are unchanged
+    :param f: Called with each key and value and returns the same key or a modified key
+    :param dct:
+    :return: A dct with keys possibly modifed but values unchanged
+    """
+    f_dict = {}
+    for k, v in dct.items():
+        f_dict[f(k, v)] = v
+    return f_dict
+
+
 def merge_dicts(*dict_args):
     """
     Given any number of dicts, shallow copy and merge into a new dict,
@@ -473,6 +495,19 @@ def merge_deep(dct1, dct2, merger=None):
     return my_merger.merge(dct1, dct2)
 
 
+def merge_all(dcts):
+    """
+        Shallow merge all the dcts
+    :param dcts:
+    :return:
+    """
+    return reduce(
+        lambda accum, dct: merge(accum, dct),
+        dict(),
+        dcts
+    )
+
+
 def merge_deep_all(dcts):
     """
         Merge deep all dicts using merge_deep
@@ -509,13 +544,22 @@ def compact(lst):
 
 def from_pairs(pairs):
     """
-        Implementation of ramda from_paris Converts a list of pairs or tuples of pairs to a dict
+        Implementation of ramda from_pairs Converts a list of pairs or tuples of pairs to a dict
     :param pairs:
     :return:
     """
     return {k: v for k, v in pairs}
 
 
+def to_pairs(dct):
+    """
+        Implementation of ramda to_pairs Converts a dict to a list of pairs
+    :param dct:
+    :return:
+    """
+    return dct.items()
+
+
 def flatten(lst):
     """
         Impemenation of ramda flatten
@@ -611,3 +655,60 @@ def to_dict_deep(obj, classkey=None):
         return data
     else:
         return obj
+
+
+def flatten_dct_until(obj, until_func, separator):
+    """
+    Flattens an objects so deep keys and array indices become concatinated strings
+    E.g. {a: {b: [1, 3]}} => {'a.b.0': 1, 'a.b.1': 2}
+    @param {Object} obj The object to flattened
+    @param {Function} until_func stop flattening a line if the this function returns false for the current key
+    @param {Object} separator Key segment separator, probably either '.' or '__'
+    @returns {Object} The 1-D version of the object
+    :param obj:
+    :return:
+    """
+    return from_pairs(_flatten_dct(obj, until_func, separator))
+
+def flatten_dct(obj, separator):
+    """
+    Flattens an objects so deep keys and array indices become concatinated strings
+    E.g. {a: {b: [1, 3]}} => {'a.b.0': 1, 'a.b.1': 2}
+    @param {Object} obj The object to flattened
+    @param {Object} separator Key segment separator, probably either '.' or '__'
+    @returns {Object} The 1-D version of the object
+    :param obj:
+    :return:
+    """
+    return from_pairs(_flatten_dct(obj, always(True), separator))
+
+def _flatten_dct(obj, until_func, separator, recurse_keys=[]):
+    """
+
+    :param obj:
+    :param until_func: Stops recursion on a certain line if the function returns false and the remaining
+    value is returned with the key
+    :param recurse_keys:
+    :return:
+    """
+    return if_else(
+        # If we have something iterable besides a string
+        isinstance((dict, list, tuple)),
+        # Then recurse on each object or array value
+        lambda o: compose(
+            flatten,
+            map_with_obj_to_values(
+                lambda k, oo: _flatten_dct(oo, until_func, separator, concat(recurse_keys, [k])) if
+                until_func(k) else
+                [[join(separator, concat(recurse_keys, [k])), oo]]
+            ),
+            # Convert lists and tuples to dict where indexes become keys
+            if_else(isinstance(dict), identity, list_to_dict)
+        )(o),
+        # If not an object return a single pair
+        lambda o: [[join(separator, recurse_keys), o]]
+    )(obj)
+
+
+def list_to_dict(lst):
+    return dict(zip(range(len(lst)), lst))
